package iobuffers;

import java.io.ByteArrayInputStream;
import java.io.DataInput;
import java.io.DataInputStream;
import java.io.IOException;


/**
 * This class implements an input stream in which the data is read
 * from a buffer and provides methods to read primitive data types.
 * It extends a <code>ByteArrayInputStream</code> and relies on a
 * <code>DataInputStream</code> to read primitive data types.
 */
public class InputBuffer extends ByteArrayInputStream implements DataInput {

    /** DataInputStream used to read primitive data types. */
    private final DataInputStream in;


    /**
     * Creates a new <code>InputBuffer</code> that uses
     * <code>buf</code> as its buffer array.
     * @param buf the input buffer.
     */
    public InputBuffer(byte[] buf) {
        super(buf);
        in = new DataInputStream(this);
    }


    /**
     * Creates a new <code>InputBuffer</code> that uses a portion of
     * <code>buf</code> as its buffer array (from <code>offset</code>
     * for <code>length</code> bytes).
     * @param buf the input buffer.
     * @param offset the offset in the buffer of the first byte to read.
     * @param length the maximum number of bytes to read from the buffer.
     */
    public InputBuffer(byte[] buf, int offset, int length) {
        super(buf, offset, length);
        in = new DataInputStream(this);
    }
    
    
    /**
     * Creates a new <code>InputBuffer</code> that uses the buffer of
     * an <code>OutputBuffer</code> as its buffer array.
     * @param buf the <code>OutputBuffer</code> from which read bytes.
     */
    public InputBuffer(OutputBuffer buf) {
        super(buf.buffer(), 0, buf.size());
        in = new DataInputStream(this);
    }


    /**
     * Gets a reference to the buffer from which data is read.
     * @return the buffer from which data is read.
     */
    public byte[] buffer() {
        return this.buf;
    }


    @Override
    public int read(byte[] b) {
        return this.read(b, 0, b.length);
    }

    
    public void readFully(byte[] b) throws IOException {
        in.readFully(b);
    }


    public void readFully(byte[] b, int off, int len) throws IOException {
        in.readFully(b, off, len);
    }


    public int skipBytes(int n) throws IOException {
        return in.skipBytes(n);
    }


    public boolean readBoolean() throws IOException {
        return in.readBoolean();
    }


    public byte readByte() throws IOException {
        return in.readByte();
    }


    public int readUnsignedByte() throws IOException {
        return in.readUnsignedByte();
    }


    public short readShort() throws IOException {
        return in.readShort();
    }


    public int readUnsignedShort() throws IOException {
        return in.readUnsignedShort();
    }


    public char readChar() throws IOException {
        return in.readChar();
    }


    public int readInt() throws IOException {
        return in.readInt();
    }


    public long readLong() throws IOException {
        return in.readLong();
    }


    public float readFloat() throws IOException {
        return in.readFloat();
    }


    public double readDouble() throws IOException {
        return in.readDouble();
    }


    @Deprecated
    public String readLine() throws IOException {
        return in.readLine();
    }


    public String readUTF() throws IOException {
        return in.readUTF();
    }
    
}
